#ifndef RANDOM_H_INCLUDED
#define RANDOM_H_INCLUDED

//! Mersenne twister implementation
#define MT_STATE_SIZE 624
#define MT_STATE_SIZE_MO 623

void mt_init(u32 seed);
u32 mt_extract_number();




//! Sha256 implementation
#define SHA256_DIGEST_SIZE 32
#define SHA256_DIGEST_SIZE_MO (SHA256_DIGEST_SIZE - 1)

struct CSha256 {
    u32 state[8];
    u64 count;
    u8 buffer[64];
};

void Sha256_Init(CSha256 *p);
void Sha256_Update(CSha256 *p, const u8 *data, u64 size);
void Sha256_Final(CSha256 *p, u8 *digest);



//! Cryptographically secure RNG (Mersenne twister -> sha256)
namespace random {
    u32 cs_rand();
    void cs_srand(u32 seed);
};


//! Helper macros
#ifdef RAND_MAX
    #undef RAND_MAX
#endif

#define RAND_MAX 0xffffffff
#define RAND_MAX_HALF 0x7fffffff
#define rand() random::cs_rand()
#define srand(seed) random::cs_srand(seed)
#define frand() ((f64)rand() / (f64)RAND_MAX)
#define frand2() (((f64)rand() / (f64)RAND_MAX_HALF) - 1.0)

inline f64 frandc(u32 number) {
    f64 ret = frand();
    for(u32 i = 0; i < number; ++i) {
        ret *= frand();
    }

    return ret;
}

#endif // RANDOM_H_INCLUDED
