
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdarg.h>
#include <memory.h>
#include <assert.h>
#include <string>

using namespace std;

#include <Types.h>
#include <Utility.h>

#ifdef _WIN32
    #define WIN32_LEAN_AND_MEAN
    #include <windows.h>
    #include <io.h>
#endif

void logPrintf(const c8 * format, ...) {
    static FILE * logFile = NULL;
    if(!logFile) {
        logFile = fopen("Log.txt", "wb");
        if(!logFile) {
            fprintf(stderr, "[ ERROR ] Unable to open logfile!\n");
            exit(EXIT_FAILURE);
        }
    }

    // print the message to the logfile
    va_list args;
    va_start(args, format);
    vfprintf(logFile, format, args);
    va_end(args);

    // print the message to stderr
    va_start(args, format);
    vfprintf(stderr, format, args);
    va_end(args);
}

void * myalloc(size_t size) {
    void * ret = malloc(size);
    if(!ret) {
        logPrintf("[ ERROR ] Unable to malloc %u bytes!", size);
        *(s32*)NULL = 0;
        exit(EXIT_FAILURE);
    }

    memset(ret, 0, size);
    return ret;
}

void myfree(void * ptr) {
    if(!ptr) {
        //logPrintf("[ ERROR ] Cannot free a NULL pointer!\n");
        //exit(EXIT_FAILURE);
    }

    free(ptr);
}

void clearScreen() {
#ifdef WIN32
    system("cls");
#else
    #warn Screen clearing not implemented on linux!
#endif
}

s32 mymkdir(const string &dirname) {
    s32 ret = 0;
#ifdef _WIN32
    ret = _mkdir(dirname.c_str());
#else
    ret = mkdir(dirname.c_str(), 0775);
#endif
    return ret;
}
